@@ -668,6 +668,10 @@ public:
   static nsresult GetElementsByClassNameHelper(nsINode* aRootNode,
                                                const nsAString& aClasses,
                                                nsIDOMNodeList** aReturn);
+
+  void MaybeInitializeFinalizeFrameLoaders();
+
+  void MaybeEndOutermostXBLUpdate();
 protected:
 
   /**
@@ -679,8 +683,6 @@ protected:
 
   void DispatchContentLoadedEvents();
 
-  void InitializeFinalizeFrameLoaders();
-
   void RetrieveRelevantHeaders(nsIChannel *aChannel);
 
   static PRBool TryChannelCharset(nsIChannel *aChannel,
@@ -797,6 +799,12 @@ protected:
 
   PRPackedBool mDelayFrameLoaderInitialization:1;
 
+  // If true, we have an input encoding.  If this is false, then the
+  // document was created entirely in memory
+  PRPackedBool mHaveInputEncoding:1;
+
+  PRPackedBool mInXBLUpdate:1;
+
   PRUint8 mXMLDeclarationBits;
 
   PRUint8 mDefaultElementType;
@@ -861,6 +869,7 @@ private:
 
   nsTArray<nsRefPtr<nsFrameLoader> > mInitializableFrameLoaders;
   nsTArray<nsRefPtr<nsFrameLoader> > mFinalizableFrameLoaders;
+  nsRefPtr<nsRunnableMethod<nsDocument> > mFrameLoaderRunner;
 };
 
 
