@@ -201,6 +201,7 @@ protected:
                         PRBool* aFailureFromContent);
 
     nsresult ApplyPersistentAttributes();
+    nsresult ApplyPersistentAttributesInternal();
     nsresult ApplyPersistentAttributesToElements(nsIRDFResource* aResource,
                                                  nsCOMArray<nsIContent>& aElements);
 
@@ -237,6 +238,9 @@ protected:
 
     static PRLogModuleInfo* gXULLog;
 
+    PRBool
+    IsCapabilityEnabled(const char* aCapabilityLabel);
+
     nsresult
     Persist(nsIContent* aElement, PRInt32 aNameSpaceID, nsIAtom* aAttribute);
 
