@@ -153,7 +153,8 @@ protected:
 #define LL_NEEDBACKUP                  0x00000400
 #define LL_INFIRSTLINE                 0x00000800
 #define LL_GOTLINEBOX                  0x00001000
-#define LL_LASTFLAG                    LL_GOTLINEBOX
+#define LL_INFIRSTLETTER               0x00002000
+#define LL_LASTFLAG                    LL_INFIRSTLETTER
 
   PRUint16 mFlags;
 
@@ -223,6 +224,14 @@ public:
     SetFlag(LL_FIRSTLETTERSTYLEOK, aSetting);
   }
 
+  PRBool GetInFirstLetter() const {
+    return GetFlag(LL_INFIRSTLETTER);
+  }
+
+  void SetInFirstLetter(PRBool aSetting) {
+    SetFlag(LL_INFIRSTLETTER, aSetting);
+  }
+
   PRBool GetInFirstLine() const {
     return GetFlag(LL_INFIRSTLINE);
   }
