@@ -164,11 +164,11 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
     mMustNotify(PR_FALSE),
     mWasAlternate(aIsAlternate),
     mAllowUnsafeRules(PR_FALSE),
+    mUseSystemPrincipal(PR_FALSE),
     mOwningElement(aOwningElement),
     mObserver(aObserver),
     mLoaderPrincipal(aLoaderPrincipal)
 {
-
   NS_PRECONDITION(mLoader, "Must have a loader!");
   NS_ADDREF(mLoader);
 }
@@ -193,11 +193,11 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
     mMustNotify(PR_FALSE),
     mWasAlternate(PR_FALSE),
     mAllowUnsafeRules(PR_FALSE),
+    mUseSystemPrincipal(PR_FALSE),
     mOwningElement(nsnull),
     mObserver(aObserver),
     mLoaderPrincipal(aLoaderPrincipal)
 {
-
   NS_PRECONDITION(mLoader, "Must have a loader!");
   NS_ADDREF(mLoader);
   if (mParentData) {
@@ -205,8 +205,12 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
     mSyncLoad = mParentData->mSyncLoad;
     mIsNonDocumentSheet = mParentData->mIsNonDocumentSheet;
     mAllowUnsafeRules = mParentData->mAllowUnsafeRules;
+    mUseSystemPrincipal = mParentData->mUseSystemPrincipal;
     ++(mParentData->mPendingChildren);
   }
+
+  NS_POSTCONDITION(!mUseSystemPrincipal || mSyncLoad,
+                   "Shouldn't use system principal for async loads");
 }
 
 SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
@@ -214,6 +218,7 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
                              nsICSSStyleSheet* aSheet,
                              PRBool aSyncLoad,
                              PRBool aAllowUnsafeRules,
+                             PRBool aUseSystemPrincipal,
                              nsICSSLoaderObserver* aObserver,
                              nsIPrincipal* aLoaderPrincipal)
   : mLoader(aLoader),
@@ -230,13 +235,16 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader,
     mMustNotify(PR_FALSE),
     mWasAlternate(PR_FALSE),
     mAllowUnsafeRules(aAllowUnsafeRules),
+    mUseSystemPrincipal(aUseSystemPrincipal),
     mOwningElement(nsnull),
     mObserver(aObserver),
     mLoaderPrincipal(aLoaderPrincipal)
 {
-
   NS_PRECONDITION(mLoader, "Must have a loader!");
   NS_ADDREF(mLoader);
+
+  NS_POSTCONDITION(!mUseSystemPrincipal || mSyncLoad,
+                   "Shouldn't use system principal for async loads");
 }
 
 SheetLoadData::~SheetLoadData()
@@ -280,7 +288,7 @@ CSSLoaderImpl::~CSSLoaderImpl(void)
   // they're all done.
 }
 
-NS_IMPL_ISUPPORTS1(CSSLoaderImpl, nsICSSLoader)
+NS_IMPL_ISUPPORTS2(CSSLoaderImpl, nsICSSLoader, nsICSSLoader_1_9_0_BRANCH)
 
 void
 CSSLoaderImpl::Shutdown()
@@ -1275,6 +1283,8 @@ CSSLoaderImpl::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
   NS_PRECONDITION(aLoadData->mURI, "Need a URI to load");
   NS_PRECONDITION(aLoadData->mSheet, "Need a sheet to load into");
   NS_PRECONDITION(aSheetState != eSheetComplete, "Why bother?");
+  NS_PRECONDITION(!aLoadData->mUseSystemPrincipal || aLoadData->mSyncLoad,
+                  "Shouldn't use system principal for async loads");
   NS_ASSERTION(mLoadingDatas.IsInitialized(), "mLoadingDatas should be initialized by now.");
 
   LOG_URI("  Load from: '%s'", aLoadData->mURI);
@@ -1308,10 +1318,16 @@ CSSLoaderImpl::LoadSheet(SheetLoadData* aLoadData, StyleSheetState aSheetState)
 
     NS_ASSERTION(channel, "NS_OpenURI lied?");
     
-    // Get the principal for this channel
+    // Get the principal for this sheet
     nsCOMPtr<nsIPrincipal> principal;
-    rv = nsContentUtils::GetSecurityManager()->
-      GetChannelPrincipal(channel, getter_AddRefs(principal));
+    if (aLoadData->mUseSystemPrincipal) {
+      rv = nsContentUtils::GetSecurityManager()->
+        GetSystemPrincipal(getter_AddRefs(principal));
+    } else {
+      rv = nsContentUtils::GetSecurityManager()->
+        GetChannelPrincipal(channel, getter_AddRefs(principal));
+    }
+
     if (NS_FAILED(rv)) {
       LOG_ERROR(("  Failed to get a principal for the sheet"));
       SheetComplete(aLoadData, rv);
@@ -1961,9 +1977,18 @@ CSSLoaderImpl::LoadChildSheet(nsICSSStyleSheet* aParentSheet,
 NS_IMETHODIMP
 CSSLoaderImpl::LoadSheetSync(nsIURI* aURL, PRBool aAllowUnsafeRules,
                              nsICSSStyleSheet** aSheet)
+{
+  return LoadSheetSync(aURL, aAllowUnsafeRules, PR_FALSE, aSheet);
+}
+
+NS_IMETHODIMP
+CSSLoaderImpl::LoadSheetSync(nsIURI* aURL, PRBool aAllowUnsafeRules,
+                             PRBool aUseSystemPrincipal,
+                             nsICSSStyleSheet** aSheet)
 {
   LOG(("CSSLoaderImpl::LoadSheetSync"));
-  return InternalLoadNonDocumentSheet(aURL, aAllowUnsafeRules, nsnull,
+  return InternalLoadNonDocumentSheet(aURL, aAllowUnsafeRules,
+                                      aUseSystemPrincipal, nsnull,
                                       aSheet, nsnull);
 }
 
@@ -1975,7 +2000,8 @@ CSSLoaderImpl::LoadSheet(nsIURI* aURL,
 {
   LOG(("CSSLoaderImpl::LoadSheet(aURL, aObserver, aSheet) api call"));
   NS_PRECONDITION(aSheet, "aSheet is null");
-  return InternalLoadNonDocumentSheet(aURL, PR_FALSE, aOriginPrincipal,
+  return InternalLoadNonDocumentSheet(aURL, PR_FALSE, PR_FALSE,
+                                      aOriginPrincipal,
                                       aSheet, aObserver);
 }
 
@@ -1985,19 +2011,23 @@ CSSLoaderImpl::LoadSheet(nsIURI* aURL,
                          nsICSSLoaderObserver* aObserver)
 {
   LOG(("CSSLoaderImpl::LoadSheet(aURL, aObserver) api call"));
-  return InternalLoadNonDocumentSheet(aURL, PR_FALSE, aOriginPrincipal,
+  return InternalLoadNonDocumentSheet(aURL, PR_FALSE, PR_FALSE,
+                                      aOriginPrincipal,
                                       nsnull, aObserver);
 }
 
 nsresult
 CSSLoaderImpl::InternalLoadNonDocumentSheet(nsIURI* aURL, 
                                             PRBool aAllowUnsafeRules,
+                                            PRBool aUseSystemPrincipal,
                                             nsIPrincipal* aOriginPrincipal,
                                             nsICSSStyleSheet** aSheet,
                                             nsICSSLoaderObserver* aObserver)
 {
   NS_PRECONDITION(aURL, "Must have a URI to load");
   NS_PRECONDITION(aSheet || aObserver, "Sheet and observer can't both be null");
+  NS_PRECONDITION(!aUseSystemPrincipal || !aObserver,
+                  "Shouldn't load system-principal sheets async");
   NS_ASSERTION(mParsingDatas.Count() == 0, "We're in the middle of a parse?");
 
   LOG_URI("  Non-document sheet uri: '%s'", aURL);
@@ -2041,7 +2071,7 @@ CSSLoaderImpl::InternalLoadNonDocumentSheet(nsIURI* aURL,
 
   SheetLoadData* data =
     new SheetLoadData(this, aURL, sheet, syncLoad, aAllowUnsafeRules,
-                      aObserver, aOriginPrincipal);
+                      aUseSystemPrincipal, aObserver, aOriginPrincipal);
 
   if (!data) {
     sheet->SetComplete();
@@ -2143,6 +2173,15 @@ nsresult NS_NewCSSLoader(nsICSSLoader** aLoader)
   return CallQueryInterface(it, aLoader);
 }
 
+nsresult NS_NewCSSLoader(nsICSSLoader_1_9_0_BRANCH** aLoader)
+{
+  CSSLoaderImpl* it = new CSSLoaderImpl();
+
+  NS_ENSURE_TRUE(it, NS_ERROR_OUT_OF_MEMORY);
+
+  return CallQueryInterface(it, aLoader);
+}
+
 PR_STATIC_CALLBACK(PLDHashOperator)
 StopLoadingSheetCallback(nsURIAndPrincipalHashKey* aKey,
                          SheetLoadData*& aData,
