@@ -21,6 +21,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Jeff Walden <jwalden+code@mit.edu>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,13 +37,17 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "nsJAR.h"
 #include "nsJARChannel.h"
 #include "nsJARProtocolHandler.h"
 #include "nsMimeTypes.h"
 #include "nsNetUtil.h"
 #include "nsInt64.h"
+#include "nsEscape.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
+#include "nsIViewSourceChannel.h"
+#include "nsChannelProperties.h"
 
 #include "nsIScriptSecurityManager.h"
 #include "nsIPrincipal.h"
@@ -51,6 +56,11 @@
 
 static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 
+// the entry for a directory will either be empty (in the case of the
+// top-level directory) or will end with a slash
+#define ENTRY_IS_DIRECTORY(_entry) \
+  ((_entry).IsEmpty() || '/' == (_entry).Last())
+
 //-----------------------------------------------------------------------------
 
 #if defined(PR_LOGGING)
@@ -75,10 +85,13 @@ public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSIINPUTSTREAM
 
-    nsJARInputThunk(nsIFile *jarFile, const nsACString &jarEntry,
+    nsJARInputThunk(nsIFile *jarFile,
+                    nsIURI* fullJarURI,
+                    const nsACString &jarEntry,
                     nsIZipReaderCache *jarCache)
         : mJarCache(jarCache)
         , mJarFile(jarFile)
+        , mFullJarURI(fullJarURI)
         , mJarEntry(jarEntry)
         , mContentLength(-1)
     {
@@ -108,6 +121,7 @@ private:
     nsCOMPtr<nsIZipReaderCache> mJarCache;
     nsCOMPtr<nsIZipReader>      mJarReader;
     nsCOMPtr<nsIFile>           mJarFile;
+    nsCOMPtr<nsIURI>            mFullJarURI;
     nsCOMPtr<nsIInputStream>    mJarStream;
     nsCString                   mJarEntry;
     PRInt32                     mContentLength;
@@ -129,16 +143,33 @@ nsJARInputThunk::EnsureJarStream()
         mJarReader = do_CreateInstance(kZipReaderCID, &rv);
         if (NS_FAILED(rv)) return rv;
 
-        rv = mJarReader->Init(mJarFile);
-        if (NS_FAILED(rv)) return rv;
-
-        rv = mJarReader->Open();
+        rv = mJarReader->Open(mJarFile);
     }
     if (NS_FAILED(rv)) return rv;
 
-    rv = mJarReader->GetInputStream(mJarEntry.get(),
-                                    getter_AddRefs(mJarStream));
-    if (NS_FAILED(rv)) return rv;
+    if (ENTRY_IS_DIRECTORY(mJarEntry)) {
+        // A directory stream also needs the Spec of the FullJarURI
+        // because is included in the stream data itself.
+
+        nsCAutoString jarDirSpec;
+        rv = mFullJarURI->GetAsciiSpec(jarDirSpec);
+        if (NS_FAILED(rv)) return rv;
+
+        rv = mJarReader->GetInputStreamWithSpec(jarDirSpec,
+                                                mJarEntry.get(),
+                                                getter_AddRefs(mJarStream));
+    }
+    else {
+        rv = mJarReader->GetInputStream(mJarEntry.get(),
+                                        getter_AddRefs(mJarStream));
+    }
+    if (NS_FAILED(rv)) {
+        // convert to the proper result if the entry wasn't found
+        // so that error pages work
+        if (rv == NS_ERROR_FILE_TARGET_DOES_NOT_EXIST)
+            rv = NS_ERROR_FILE_NOT_FOUND;
+        return rv;
+    }
 
     // ask the JarStream for the content length
     mJarStream->Available((PRUint32 *) &mContentLength);
@@ -189,6 +220,9 @@ nsJARInputThunk::IsNonBlocking(PRBool *nonBlocking)
 }
 
 //-----------------------------------------------------------------------------
+// nsJARChannel
+//-----------------------------------------------------------------------------
+
 
 nsJARChannel::nsJARChannel()
     : mContentLength(-1)
@@ -217,19 +251,23 @@ nsJARChannel::~nsJARChannel()
     NS_RELEASE(handler); // NULL parameter
 }
 
-NS_IMPL_ISUPPORTS7(nsJARChannel,
-                   nsIRequest,
-                   nsIChannel,
-                   nsIStreamListener,
-                   nsIRequestObserver,
-                   nsIDownloadObserver,
-                   nsIJARChannel,
-                   nsIJARChannel_MOZILLA_1_8_BRANCH)
+NS_IMPL_ISUPPORTS_INHERITED6(nsJARChannel,
+                             nsHashPropertyBag,
+                             nsIRequest,
+                             nsIChannel,
+                             nsIStreamListener,
+                             nsIRequestObserver,
+                             nsIDownloadObserver,
+                             nsIJARChannel)
 
 nsresult 
 nsJARChannel::Init(nsIURI *uri)
 {
     nsresult rv;
+    rv = nsHashPropertyBag::Init();
+    if (NS_FAILED(rv))
+        return rv;
+
     mJarURI = do_QueryInterface(uri, &rv);
     if (NS_FAILED(rv))
         return rv;
@@ -263,7 +301,7 @@ nsJARChannel::CreateJarInput(nsIZipReaderCache *jarCache)
     nsresult rv = mJarFile->Clone(getter_AddRefs(clonedFile));
     if (NS_FAILED(rv)) return rv;
 
-    mJarInput = new nsJARInputThunk(clonedFile, mJarEntry, jarCache);
+    mJarInput = new nsJARInputThunk(clonedFile, mJarURI, mJarEntry, jarCache);
     if (!mJarInput)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(mJarInput);
@@ -284,6 +322,12 @@ nsJARChannel::EnsureJarInput(PRBool blocking)
     rv = mJarURI->GetJAREntry(mJarEntry);
     if (NS_FAILED(rv)) return rv;
 
+    // The name of the JAR entry must not contain URL-escaped characters:
+    // we're moving from URL domain to a filename domain here. nsStandardURL
+    // does basic escaping by default, which breaks reading zipped files which
+    // have e.g. spaces in their filenames.
+    NS_UnescapeURL(mJarEntry);
+
     // try to get a nsIFile directly from the url, which will often succeed.
     {
         nsCOMPtr<nsIFileURL> fileURL = do_QueryInterface(mJarBaseURI);
@@ -531,26 +575,31 @@ nsJARChannel::GetContentType(nsACString &result)
         //
         // generate content type and set it
         //
-        if (mJarEntry.IsEmpty()) {
-            LOG(("mJarEntry is empty!\n"));
-            return NS_ERROR_NOT_AVAILABLE;
-        }
-    
         const char *ext = nsnull, *fileName = mJarEntry.get();
         PRInt32 len = mJarEntry.Length();
-        for (PRInt32 i = len-1; i >= 0; i--) {
-            if (fileName[i] == '.') {
-                ext = &fileName[i + 1];
-                break;
+
+        // check if we're displaying a directory
+        // mJarEntry will be empty if we're trying to display
+        // the topmost directory in a zip, e.g. jar:foo.zip!/
+        if (ENTRY_IS_DIRECTORY(mJarEntry)) {
+            mContentType.AssignLiteral(APPLICATION_HTTP_INDEX_FORMAT);
+        }
+        else {
+            // not a directory, take a guess by its extension
+            for (PRInt32 i = len-1; i >= 0; i--) {
+                if (fileName[i] == '.') {
+                    ext = &fileName[i + 1];
+                    break;
+                }
             }
+            if (ext) {
+                nsIMIMEService *mimeServ = gJarHandler->MimeService();
+                if (mimeServ)
+                    mimeServ->GetTypeFromExtension(nsDependentCString(ext), mContentType);
+            }
+            if (mContentType.IsEmpty())
+                mContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
         }
-        if (ext) {
-            nsIMIMEService *mimeServ = gJarHandler->MimeService();
-            if (mimeServ)
-                mimeServ->GetTypeFromExtension(nsDependentCString(ext), mContentType);
-        }
-        if (mContentType.IsEmpty())
-            mContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
     }
     result = mContentType;
     return NS_OK;
@@ -632,6 +681,7 @@ nsJARChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
 {
     LOG(("nsJARChannel::AsyncOpen [this=%x]\n", this));
 
+    NS_ENSURE_ARG_POINTER(listener);
     NS_ENSURE_TRUE(!mIsPending, NS_ERROR_IN_PROGRESS);
 
     mJarFile = nsnull;
@@ -684,11 +734,8 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
 {
     nsresult rv;
 
-    // Grab the security info from our base channel
     nsCOMPtr<nsIChannel> channel(do_QueryInterface(request));
     if (channel) {
-        channel->GetSecurityInfo(getter_AddRefs(mSecurityInfo));
-
         PRUint32 loadFlags;
         channel->GetLoadFlags(&loadFlags);
         if (loadFlags & LOAD_REPLACE) {
@@ -702,43 +749,57 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
             rv = channel->GetURI(getter_AddRefs(innerURI));
             if (NS_SUCCEEDED(rv)) {
                 nsCOMPtr<nsIJARURI> newURI;
-                nsCOMPtr<nsIJARURI_MOZILLA_1_8_BRANCH> jarURIBranch =
-                    do_QueryInterface(mJarURI);
-                rv = jarURIBranch->CloneWithJARFile(innerURI,
-                                                    getter_AddRefs(newURI));
+                rv = mJarURI->CloneWithJARFile(innerURI,
+                                               getter_AddRefs(newURI));
                 if (NS_SUCCEEDED(rv)) {
                     mJarURI = newURI;
                 }
             }
-            status = rv;
+            if (NS_SUCCEEDED(status)) {
+                status = rv;
+            }
         }
+    }
+
+    if (NS_SUCCEEDED(status) && channel) {
+        nsCAutoString header;
+        // Grab the security info from our base channel
+        channel->GetSecurityInfo(getter_AddRefs(mSecurityInfo));
 
         nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(channel));
         if (httpChannel) {
             // We only want to run scripts if the server really intended to
             // send us a JAR file.  Check the server-supplied content type for
             // a JAR type.
-            nsCAutoString header;
             httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Content-Type"),
                                            header);
-
             nsCAutoString contentType;
             nsCAutoString charset;
             NS_ParseContentType(header, contentType, charset);
-
-            mIsUnsafe = !contentType.EqualsLiteral("application/java-archive") &&
-                        !contentType.EqualsLiteral("application/x-jar");
+            nsCAutoString channelContentType;
+            channel->GetContentType(channelContentType);
+            mIsUnsafe = !(contentType.Equals(channelContentType) &&
+                          (contentType.EqualsLiteral("application/java-archive") ||
+                           contentType.EqualsLiteral("application/x-jar")));
+            rv = httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("Content-Disposition"),
+                                                header);
+            if (NS_SUCCEEDED(rv))
+                SetPropertyAsACString(NS_CHANNEL_PROP_CONTENT_DISPOSITION, header);
         } else {
-            nsCOMPtr<nsIJARChannel_MOZILLA_1_8_BRANCH> innerJARChannel(do_QueryInterface(channel));
+            nsCOMPtr<nsIJARChannel> innerJARChannel(do_QueryInterface(channel));
             if (innerJARChannel) {
                 PRBool unsafe;
                 innerJARChannel->GetIsUnsafe(&unsafe);
                 mIsUnsafe = unsafe;
             }
+            // Soon-to-be common way to get Disposition: right now only nsIJARChannel
+            rv = NS_GetContentDisposition(request, header);
+            if (NS_SUCCEEDED(rv))
+                SetPropertyAsACString(NS_CHANNEL_PROP_CONTENT_DISPOSITION, header);
         }
     }
 
-    if (mIsUnsafe) {
+    if (NS_SUCCEEDED(status) && mIsUnsafe) {
         PRBool allowUnpack = PR_FALSE;
 
         nsCOMPtr<nsIPrefBranch> prefs =
@@ -752,6 +813,14 @@ nsJARChannel::OnDownloadComplete(nsIDownloader *downloader,
         }
     }
 
+    if (NS_SUCCEEDED(status)) {
+        // Refuse to unpack view-source: jars even if open-unsafe-types is set.
+        nsCOMPtr<nsIViewSourceChannel> viewSource = do_QueryInterface(channel);
+        if (viewSource) {
+            status = NS_ERROR_UNSAFE_CONTENT_TYPE;
+        }
+    }
+
     if (NS_SUCCEEDED(status)) {
         mJarFile = file;
     
