@@ -403,13 +403,13 @@ nsDiskCacheDevice::FindEntry(nsCString * key, PRBool *collision)
 
     *collision = PR_FALSE;
 
-#if DEBUG  /* because we shouldn't be called for active entries */
     binding = mBindery.FindActiveBinding(hashNumber);
-    NS_ASSERTION(!binding || strcmp(binding->mCacheEntry->Key()->get(), key->get()) != 0,
-                 "FindEntry() called for a bound entry.");
+    if (binding && PL_strcmp(binding->mCacheEntry->Key()->get(), key->get()) != 0) {
+        *collision = PR_TRUE;
+        return nsnull;
+    }
     binding = nsnull;
-#endif
-    
+
     // lookup hash number in cache map
     nsresult rv = mCacheMap.FindRecord(hashNumber, &record);
     if (NS_FAILED(rv))  return nsnull;  // XXX log error?
