@@ -571,6 +571,7 @@ class nsXULAppInfo : public nsIXULAppInfo,
 #endif
 #ifdef MOZ_CRASHREPORTER
                      public nsICrashReporter,
+                     public nsICrashReporter_MOZILLA_1_9_BRANCH,
 #endif
                      public nsIXULRuntime
                      
@@ -581,6 +582,7 @@ public:
   NS_DECL_NSIXULRUNTIME
 #ifdef MOZ_CRASHREPORTER
   NS_DECL_NSICRASHREPORTER
+  NS_DECL_NSICRASHREPORTER_MOZILLA_1_9_BRANCH
 #endif
 #ifdef XP_WIN
   NS_DECL_NSIWINAPPHELPER
@@ -597,6 +599,7 @@ NS_INTERFACE_MAP_BEGIN(nsXULAppInfo)
 #endif
 #ifdef MOZ_CRASHREPORTER
   NS_INTERFACE_MAP_ENTRY(nsICrashReporter)
+  NS_INTERFACE_MAP_ENTRY(nsICrashReporter_MOZILLA_1_9_BRANCH)
 #endif
   NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsIXULAppInfo, gAppData)
 NS_INTERFACE_MAP_END
@@ -836,6 +839,12 @@ nsXULAppInfo::AnnotateCrashReport(const nsACString& key,
   return CrashReporter::AnnotateCrashReport(key, data);
 }
 
+NS_IMETHODIMP
+nsXULAppInfo::AppendAppNotesToCrashReport(const nsACString& data)
+{
+  return CrashReporter::AppendAppNotesToCrashReport(data);
+}
+
 NS_IMETHODIMP
 nsXULAppInfo::WriteMinidumpForException(void* aExceptionInfo)
 {
@@ -845,7 +854,17 @@ nsXULAppInfo::WriteMinidumpForException(void* aExceptionInfo)
   return NS_ERROR_NOT_IMPLEMENTED;
 #endif
 }
+
+NS_IMETHODIMP
+nsXULAppInfo::AppendObjCExceptionInfoToAppNotes(void* aException)
+{
+#ifdef XP_MACOSX
+  return CrashReporter::AppendObjCExceptionInfoToAppNotes(aException);
+#else
+  return NS_ERROR_NOT_IMPLEMENTED;
 #endif
+}
+#endif // MOZ_CRASHREPORTER
 
 static const nsXULAppInfo kAppInfo;
 static NS_METHOD AppInfoConstructor(nsISupports* aOuter,
@@ -2465,6 +2484,9 @@ XRE_main(int argc, char* argv[], const nsXREAppData* aAppData)
       PR_SetEnv(expr);
     // We intentionally leak |expr| here since it is required by PR_SetEnv.
   }
+
+  // Suppress atk-bridge init at startup, it works after GNOME 2.24.2
+  PR_SetEnv("NO_AT_BRIDGE=1");
 #endif
 
   // Unbuffer stdout, needed for tinderbox tests.
