@@ -67,6 +67,8 @@
 #include "nsIWindowWatcher.h"
 #include "nsIAuthPrompt.h"
 #include "nsIWindowMediator.h"
+#include "nsIDocument.h"
+#include "nsIDOMDocument.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
@@ -113,6 +115,7 @@ nsXPInstallManager::nsXPInstallManager()
 
 nsXPInstallManager::~nsXPInstallManager()
 {
+    NS_ASSERT_OWNINGTHREAD(nsXPInstallManager);
     NS_ASSERTION(!mTriggers, "Shutdown not called, triggers still alive");
 }
 
@@ -133,8 +136,8 @@ NS_INTERFACE_MAP_BEGIN(nsXPInstallManager)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsISupportsWeakReference)
 NS_INTERFACE_MAP_END
 
-NS_IMPL_ADDREF(nsXPInstallManager)
-NS_IMPL_RELEASE(nsXPInstallManager)
+NS_IMPL_THREADSAFE_ADDREF(nsXPInstallManager)
+NS_IMPL_THREADSAFE_RELEASE(nsXPInstallManager)
 
 NS_IMETHODIMP
 nsXPInstallManager::InitManagerFromChrome(const PRUnichar **aURLs,
@@ -232,6 +235,17 @@ nsXPInstallManager::InitManager(nsIDOMWindowInternal* aParentWindow, nsXPITrigge
 
     mParentWindow = aParentWindow;
 
+    // Attempt to find a load group, continue if we can't find one though
+    if (aParentWindow) {
+        nsCOMPtr<nsIDOMDocument> domdoc;
+        rv = aParentWindow->GetDocument(getter_AddRefs(domdoc));
+        if (NS_SUCCEEDED(rv) && domdoc) {
+            nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
+            if (doc)
+                mLoadGroup = doc->GetDocumentLoadGroup();
+        }
+    }
+
     // Start downloading initial chunks looking for signatures,
     mOutstandingCertLoads = mTriggers->Size();
 
@@ -241,7 +255,7 @@ nsXPInstallManager::InitManager(nsIDOMWindowInternal* aParentWindow, nsXPITrigge
     NS_NewURI(getter_AddRefs(uri), NS_ConvertUTF16toUTF8(item->mURL));
     nsCOMPtr<nsIStreamListener> listener = new CertReader(uri, nsnull, this);
     if (listener)
-        rv = NS_OpenURI(listener, nsnull, uri);
+        rv = NS_OpenURI(listener, nsnull, uri, nsnull, mLoadGroup);
     else
         rv = NS_ERROR_OUT_OF_MEMORY;
 
@@ -880,7 +894,7 @@ NS_IMETHODIMP nsXPInstallManager::DownloadNext()
                 {
                     nsCOMPtr<nsIChannel> channel;
 
-                    rv = NS_NewChannel(getter_AddRefs(channel), pURL, nsnull, nsnull, this);
+                    rv = NS_NewChannel(getter_AddRefs(channel), pURL, nsnull, mLoadGroup, this);
                     if (NS_SUCCEEDED(rv))
                     {
                         rv = channel->AsyncOpen(this, nsnull);
@@ -1119,6 +1133,9 @@ nsXPInstallManager::OnStartRequest(nsIRequest* request, nsISupports *ctxt)
         }
     }
 
+    if (mLoadGroup)
+        mLoadGroup->RemoveRequest(request, nsnull, NS_BINDING_RETARGETED);
+
     NS_ASSERTION( mItem && mItem->mFile, "XPIMgr::OnStartRequest bad state");
     if ( mItem && mItem->mFile )
     {
@@ -1143,6 +1160,7 @@ nsXPInstallManager::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
     {
 
         case NS_BINDING_SUCCEEDED:
+            NS_ASSERTION( mItem->mOutStream, "XPIManager: output stream doesn't exist");
             rv = NS_OK;
             break;
 
@@ -1159,7 +1177,6 @@ nsXPInstallManager::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
     }
 
     NS_ASSERTION( mItem, "Bad state in XPIManager");
-    NS_ASSERTION( mItem->mOutStream, "XPIManager: output stream doesn't exist");
     if ( mItem && mItem->mOutStream )
     {
         mItem->mOutStream->Close();
@@ -1371,7 +1388,7 @@ nsXPInstallManager::OnCertAvailable(nsIURI *aURI,
         return OnCertAvailable(uri, context, NS_ERROR_FAILURE, nsnull);
 
     NS_ADDREF(listener);
-    nsresult rv = NS_OpenURI(listener, nsnull, uri);
+    nsresult rv = NS_OpenURI(listener, nsnull, uri, nsnull, mLoadGroup);
 
     NS_ASSERTION(NS_SUCCEEDED(rv), "OpenURI failed");
     NS_RELEASE(listener);
